/*
 *  linux/drivers/block/dlight.c
 *
 *  Originally: linux/kernel/blk_drv/ramdisk.c  
 *  Written by Theodore Ts'o, 12/2/91
 *
 * Modifications by Fred N. van Kempen to allow for bootable root
 * disks (which are used in LINUX/Pro).  Also some cleanups.  03/03/93
 *
 *   Dave Woodhouse (D.W.Woodhouse@nortel.co.uk / dwmw2@cam.ac.uk) 10/7/97
 *
 */

#define ROMDISK_MAJOR 0x40

#include <linux/module.h>
#include <linux/sched.h>
#include <linux/minix_fs.h>
#include <linux/ext2_fs.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/mm.h>

#include <asm/system.h>
#include <asm/segment.h>
#include <asm/io.h>

#define MAJOR_NR  ROMDISK_MAJOR

#include <linux/blk.h>


static int romd_release(struct inode * inode, struct file * filp);
static int romd_open(struct inode * inode, struct file * filp);
long romd_init(void);

#define NUM_ROMDISKS 1



static u_char	*romd_start=(u_char *)0xe0000;
static int	romd_length = 0x340000;
static int romd_blocksize=0x200;

static struct file_operations romd_fops = {
	NULL,			/* lseek - default */
	block_read,		/* read - general block-dev read */
	NULL,		        /* write - general block-dev write */
	NULL,			/* readdir - bad */
	NULL,			/* select */
	NULL,			/* ioctl */
	NULL,			/* mmap */

#ifndef MODULE
	NULL,			/* no special open code */
        NULL,                   /* no special release code... */
#else
	romd_open,              /* module needs to increment use count */
	NULL,                   /* flush */
        romd_release,           /* module needs to decrement use count */
#endif
	block_fsync		/* fsync */
};


long romd_init()
{

        if (register_blkdev(MAJOR_NR,"romd",&romd_fops)) {
                printk("ROMDISK: Unable to get major %d.\n", MAJOR_NR);
                return 1;
        }
        blk_dev[MAJOR_NR].request_fn = DEVICE_REQUEST;
	romd_start=phys_to_virt(0xe0000);
        blksize_size[MAJOR_NR] = &romd_blocksize;

        return(0);
}



static void romd_request(void)
{
	int	len;
	char	*addr;
	u_short page;
	u_long sector=CURRENT->sector/*+258*/;
repeat:
	INIT_REQUEST;
	printk("sector %lu, ",sector);
	page=sector >> 4;
	if (page > 0x1f0)
	  {
	    end_request(0);
	    goto repeat;
	  }
	page += 0x10;

	writeb ((u_char)(page & 0xff), romd_start + 0x2000);
	writeb ((u_char)(page >> 8), romd_start + 0x2002);
	printk("page %u, ",page);
	addr = romd_start + ((sector & 15) << 9);

	printk("addr %p, ",addr);

	len = CURRENT->current_nr_sectors << 9;
	printk("len %u\n",len);
	if (((u_long)addr + len) > (u_long)romd_start + 0x2000)
	  {
	    end_request(0);
	    goto repeat;
	  }

	if ((MINOR(CURRENT->rq_dev) != 0) ||
	    ((u_long)addr+len > (u_long)romd_start+romd_length)) {
	  printk ("reject 1: addr+len = %lX, romd_start+romd_length = %lX\n",
		  (unsigned long) addr + (unsigned long) len,
		  (unsigned long)romd_start + (unsigned long)romd_length);
		end_request(0);
		goto repeat;
	}
	if (CURRENT-> cmd == WRITE) {
		(void ) memcpy(addr,
			      CURRENT->buffer,
			      len);
	} else if (CURRENT->cmd == READ) {
		(void) memcpy(CURRENT->buffer, 
			      addr,
			      len);
	} else
		panic("ROMDISK: unknown ROM disk command !\n");
	end_request(1);
	goto repeat;
}





#ifdef MODULE
static int romd_release(struct inode * inode, struct file * filp)
{
        MOD_DEC_USE_COUNT;
	return 0;
}



int init_module(void)
{
        int error = romd_init();
        if (!error)
                printk(KERN_INFO "ROMDISK: Loaded as module, major %d.\n", ROMDISK_MAJOR);
        return error;
}




/* Before freeing the module, invalidate all of the protected buffers! */
void cleanup_module(void)
{
	invalidate_buffers(MKDEV(MAJOR_NR, 0));

        unregister_blkdev( MAJOR_NR, "romd" );
        blk_dev[MAJOR_NR].request_fn = 0;
}

static int romd_open(struct inode * inode, struct file * filp)
{
        if (DEVICE_NR(inode->i_rdev) >= NUM_ROMDISKS)
                return -ENXIO;

        MOD_INC_USE_COUNT;

        return 0;
}


#endif  /* MODULE */














